package SPVM::Core::Object::Array::Int;

use base 'SPVM::Core::Object::Array';

1;

=head1 NAME

SPVM::Core::Object::Array::Int - Int Array Object

=head1 METHODS

Method List:

=over 2

=item * new_len

=item * set_elements

=item * set_elements_range

=item * set_data

=item * set_data_range

=item * set

=item * get

=item * get_elements

=item * get_elements_range

=item * to_data

=item * to_data_range

=back

If you know Method Detail, see L<SPVM::Core::Document::Function>.
