package SPVM::Core::Object::Package;

use base 'SPVM::Core::Object';

1;

=head1 NAME

SPVM::Core::Object::Package - Packaged based object

=head1 METHODS

Method List:

=over 2

=item * new

=item * set

=item * get

=back

If you know Method Detail, see L<SPVM::Core::Document::Function>.
