package SPVM::Short;

1;

=head1 NAME

SPVM::Short - Short Utility

=head1 CLASS METHODS

=head2 MIN_VALUE

B<sub MIN_VALUE : short ($class : class)>

A constant holding the minimum value a short can have, C<-32768>.

=head2 MAX_VALUE

B<sub MAX_VALUE : short ($class : class)>

A constant holding the maximum value a short can have, C<32767>.

=head2 BYTES

B<sub BYTES : int ($class : class)>

The number of shorts used to represent a short value in two's complement binary form, C<2>.

=head2 SIZE

B<sub SIZE : int ($class : class)>

The number of bits used to represent a short value in two's complement binary form, C<16>.
